#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 音频分析示例

这个示例展示了如何使用WavX库分析音频文件的振幅信息
"""

import os
import sys
import argparse

# 添加父目录到Python路径，以便可以导入wavx
# 注意：安装库后不需要这一步
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import wavx


def main():
    """主函数，解析命令行参数并分析音频文件"""
    parser = argparse.ArgumentParser(description='分析音频文件的振幅信息')
    parser.add_argument('audio_file', help='音频文件路径')
    parser.add_argument('--window', type=float, default=50.0, help='RMS窗口大小(毫秒)')
    parser.add_argument('--no-dc', action='store_false', dest='consider_dc', help='不考虑DC偏移')
    args = parser.parse_args()
    
    try:
        # 分析音频文件
        print(f"分析音频文件: {args.audio_file}")
        print(f"RMS窗口大小: {args.window} ms")
        print(f"考虑DC偏移: {args.consider_dc}")
        print("正在分析...")
        
        # 调用分析函数
        amplitude_info = wavx.analysis.amplitude.analyze_amplitude(
            args.audio_file,
            rms_window_ms=args.window,
            consider_dc=args.consider_dc
        )
        
        # 打印结果
        wavx.analysis.amplitude.print_amplitude_info(amplitude_info)
        
        # 如果需要，也可以直接访问特定信息
        print("\n特定信息示例:")
        print(f"峰值幅度: {amplitude_info['peak_amplitude']} dB")
        print(f"响度 (LUFS): {amplitude_info['loudness_lufs']} LUFS")
        
    except Exception as e:
        print(f"错误: {e}")
        return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main()) 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
测试WavX库的RMS标准化功能
将指定的音频文件标准化到-30dB FS
"""

import os
import sys
import time

# 添加当前目录到Python路径，以便可以导入wavx
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import wavx

input_file = r"C:\Users\admin\Desktop\250228_QUEST\voice\wanting\250226_ba_la_300_500ms.wav"
output_file = r"C:\Users\admin\Desktop\250228_QUEST\voice\wanting\250226_ba_la_300_500ms_30ei.wav"
target_rms_db = -30.0
original_info = wavx.analysis.amplitude.analyze_amplitude(input_file)
normalization_info = wavx.processing.normalization.normalize_to_target(
        input_file=input_file,
        output_file=output_file,
        target_rms_db=target_rms_db,
        reference_type="square"
)
    
# 打印标准化信息
print(f"\n音频 '{os.path.basename(input_file)}' 已从 {normalization_info['original_rms_db']:.2f} dB FS 调整到 {normalization_info['processed_rms_db']:.2f} dB FS")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 命令行界面

提供命令行访问WavX库功能的入口点
"""

import argparse
import sys
from typing import List, Optional

from . import __version__
from .analysis import amplitude
from .processing import normalization


def main(args: Optional[List[str]] = None) -> int:
    """
    命令行入口函数
    
    参数:
        args: 命令行参数列表。如果为None，则使用sys.argv
        
    返回:
        int: 退出码。0表示成功，非零表示错误
    """
    parser = argparse.ArgumentParser(
        description=f"WavX v{__version__} - 音频分析和处理工具库",
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
    parser.add_argument(
        '--version', action='version',
        version=f'WavX {__version__}'
    )
    
    # 创建子命令
    subparsers = parser.add_subparsers(dest='command', help='可用命令')
    
    # 振幅分析命令
    amplitude_parser = subparsers.add_parser('amplitude', help='分析音频文件的振幅信息')
    amplitude_parser.add_argument('audio_file', help='要分析的音频文件路径')
    amplitude_parser.add_argument('--window', type=float, default=50.0, help='RMS窗口大小(毫秒)')
    amplitude_parser.add_argument('--no-dc', action='store_false', dest='consider_dc', help='不考虑DC偏移')
    
    # RMS标准化命令
    normalize_parser = subparsers.add_parser('normalize', help='将音频文件标准化到指定的RMS电平')
    normalize_parser.add_argument('input_file', help='输入音频文件路径')
    normalize_parser.add_argument('output_file', help='输出音频文件路径')
    normalize_parser.add_argument('--target', '-t', type=float, default=-20.0, help='目标RMS电平(dB FS)，默认为-20dB')
    normalize_parser.add_argument('--reference', '-r', choices=['square', 'sine'], default='square', 
                                 help='参考信号类型，可选"square"或"sine"，默认为"square"')
    normalize_parser.add_argument('--freq', '-f', type=float, default=1000.0, help='参考信号频率(Hz)，默认为1000Hz')
    normalize_parser.add_argument('--duration', '-d', type=float, default=1.0, help='参考信号持续时间(秒)，默认为1秒')
    
    # 解析参数
    parsed_args = parser.parse_args(args)
    
    # 如果没有提供命令，显示帮助信息
    if not parsed_args.command:
        parser.print_help()
        return 0
    
    # 处理命令
    if parsed_args.command == 'amplitude':
        try:
            # 分析音频文件
            print(f"分析音频文件: {parsed_args.audio_file}")
            print(f"RMS窗口大小: {parsed_args.window} ms")
            print(f"考虑DC偏移: {parsed_args.consider_dc}")
            print("正在分析...")
            
            # 调用分析函数
            info = amplitude.analyze_amplitude(
                parsed_args.audio_file,
                rms_window_ms=parsed_args.window,
                consider_dc=parsed_args.consider_dc
            )
            
            # 打印结果
            amplitude.print_amplitude_info(info)
            
        except Exception as e:
            print(f"错误: {e}", file=sys.stderr)
            return 1
    
    elif parsed_args.command == 'normalize':
        try:
            # 标准化音频文件
            print(f"正在将音频文件 '{parsed_args.input_file}' 标准化到 {parsed_args.target} dB FS...")
            print(f"使用参考信号: {parsed_args.reference}波 {parsed_args.freq} Hz")
            print(f"输出文件: {parsed_args.output_file}")
            
            # 调用标准化函数
            info = normalization.normalize_to_target(
                input_file=parsed_args.input_file,
                output_file=parsed_args.output_file,
                target_rms_db=parsed_args.target,
                reference_type=parsed_args.reference,
                reference_duration=parsed_args.duration,
                reference_freq=parsed_args.freq
            )
            
            # 打印结果
            normalization.print_normalization_info(info)
            
        except Exception as e:
            print(f"错误: {e}", file=sys.stderr)
            return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main())


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX - 音频分析和处理工具库

WavX是一个模块化的音频处理库，提供各种声学分析和处理功能。
"""

__version__ = '0.1.2'
__author__ = 'Chord'

# 导入子模块
from . import analysis
from . import utils
from . import processing

# 设置日志
import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
音频标准化模块

提供音频电平标准化和调整功能
"""

import os
import numpy as np
import soundfile as sf
import librosa
import warnings
from typing import Dict, Any, Optional, Union, Tuple

from ..analysis import amplitude


def normalize_rms(
    audio_data: np.ndarray,
    target_rms_db: float = -20.0,
    current_rms_db: Optional[float] = None,
) -> np.ndarray:
    """
    将音频数据标准化到指定的RMS电平

    参数:
        audio_data (np.ndarray): 音频数据数组
        target_rms_db (float): 目标RMS电平，单位dB
        current_rms_db (Optional[float]): 当前RMS电平，如果为None则会自动计算

    返回:
        np.ndarray: 标准化后的音频数据
    """
    # 确保音频是二维数组 [channels, samples]
    if audio_data.ndim == 1:
        audio_data = audio_data.reshape(1, -1)
    
    # 如果未提供当前RMS，则计算
    if current_rms_db is None:
        # 计算当前RMS
        rms = np.sqrt(np.mean(audio_data**2))
        current_rms_db = 20 * np.log10(rms) if rms > 0 else -100.0
    
    # 计算增益
    gain_db = target_rms_db - current_rms_db
    gain_linear = 10 ** (gain_db / 20.0)
    
    # 应用增益
    normalized_audio = audio_data * gain_linear
    
    # 防止失真，如果最大值大于1.0，则缩放
    max_val = np.max(np.abs(normalized_audio))
    if max_val > 0.99:
        normalized_audio = normalized_audio * (0.99 / max_val)
    
    return normalized_audio


def reference_sine_wave(
    duration: float = 1.0,
    amplitude: float = 1.0,
    freq: float = 1000.0,
    sample_rate: int = 44100,
) -> np.ndarray:
    """
    生成参考正弦波

    参数:
        duration (float): 持续时间，单位秒
        amplitude (float): 振幅，1.0代表满刻度
        freq (float): 频率，单位Hz
        sample_rate (int): 采样率，单位Hz

    返回:
        np.ndarray: 正弦波数据 [1, samples]
    """
    t = np.linspace(0, duration, int(duration * sample_rate), endpoint=False)
    sine_wave = amplitude * np.sin(2 * np.pi * freq * t)
    return sine_wave.reshape(1, -1)


def reference_square_wave(
    duration: float = 1.0,
    amplitude: float = 1.0,
    freq: float = 1000.0,
    sample_rate: int = 44100,
) -> np.ndarray:
    """
    生成参考方波

    参数:
        duration (float): 持续时间，单位秒
        amplitude (float): 振幅，1.0代表满刻度
        freq (float): 频率，单位Hz
        sample_rate (int): 采样率，单位Hz

    返回:
        np.ndarray: 方波数据 [1, samples]
    """
    t = np.linspace(0, duration, int(duration * sample_rate), endpoint=False)
    square_wave = amplitude * np.sign(np.sin(2 * np.pi * freq * t))
    return square_wave.reshape(1, -1)


def normalize_to_target(
    input_file: str,
    output_file: str,
    target_rms_db: float = -20.0,
    reference_type: str = "square",
    reference_duration: float = 1.0,
    reference_freq: float = 1000.0,
) -> Dict[str, Any]:
    """
    将音频文件标准化到指定的RMS电平，并保存结果

    参数:
        input_file (str): 输入音频文件路径
        output_file (str): 输出音频文件路径
        target_rms_db (float): 目标RMS电平，单位dB
        reference_type (str): 参考信号类型，可选 "square" 或 "sine"
        reference_duration (float): 参考信号持续时间，单位秒
        reference_freq (float): 参考信号频率，单位Hz

    返回:
        Dict[str, Any]: 包含标准化信息的字典
    """
    # 读取音频文件
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        y, sr = librosa.load(input_file, sr=None, mono=False)
    
    # 确保y是二维数组 [channels, samples]
    if y.ndim == 1:
        y = y.reshape(1, -1)
    
    # 获取原始音频的RMS信息
    original_info = amplitude.analyze_amplitude(input_file)
    original_rms_db = original_info["total_rms_amplitude"]
    
    # 创建参考信号
    if reference_type.lower() == "sine":
        ref_signal = reference_sine_wave(
            duration=reference_duration,
            amplitude=1.0,
            freq=reference_freq,
            sample_rate=sr
        )
    else:  # 默认使用方波
        ref_signal = reference_square_wave(
            duration=reference_duration,
            amplitude=1.0,
            freq=reference_freq,
            sample_rate=sr
        )
    
    # 标准化参考信号到目标RMS电平
    ref_rms = np.sqrt(np.mean(ref_signal**2))
    ref_rms_db = 20 * np.log10(ref_rms) if ref_rms > 0 else -100
    normalized_ref = normalize_rms(ref_signal, target_rms_db, ref_rms_db)
    
    # 标准化音频到与参考信号相同的RMS电平
    normalized_audio = normalize_rms(y, target_rms_db, original_rms_db)
    
    # 保存输出文件
    sf_info = sf.info(input_file)
    sf.write(
        output_file,
        normalized_audio.T,
        sr,
        subtype=sf_info.subtype
    )
    
    # 分析处理后的音频
    processed_info = amplitude.analyze_amplitude(output_file)
    processed_rms_db = processed_info["total_rms_amplitude"]
    
    # 整合结果信息
    result = {
        "input_file": input_file,
        "output_file": output_file,
        "original_rms_db": original_rms_db,
        "target_rms_db": target_rms_db,
        "processed_rms_db": processed_rms_db,
        "reference_type": reference_type,
        "reference_freq": reference_freq,
        "sample_rate": sr,
        "channels": y.shape[0],
        "duration": y.shape[1] / sr,
    }
    
    return result


def print_normalization_info(info: Dict[str, Any]) -> None:
    """
    打印标准化处理信息

    参数:
        info (Dict[str, Any]): 由normalize_to_target返回的信息字典
    """
    print("===== 音频RMS标准化 =====")
    print(f"输入文件: {info['input_file']}")
    print(f"输出文件: {info['output_file']}")
    print(f"原始RMS电平: {info['original_rms_db']:.2f} dB FS")
    print(f"目标RMS电平: {info['target_rms_db']:.2f} dB FS")
    print(f"处理后RMS电平: {info['processed_rms_db']:.2f} dB FS")
    print(f"\n音频 '{os.path.basename(info['input_file'])}' 已从 {info['original_rms_db']:.2f} dB FS 调整到 {info['processed_rms_db']:.2f} dB FS")
    print(f"参考信号类型: {info['reference_type']}")
    print(f"参考信号频率: {info['reference_freq']} Hz")
    print(f"采样率: {info['sample_rate']} Hz")
    print(f"声道数: {info['channels']}")
    print(f"持续时间: {info['duration']:.2f} 秒")
    print("=========================") 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
音频振幅分析模块

提供音频文件的幅度、响度和相关声学参数的分析功能
"""

import numpy as np
import soundfile as sf
import librosa
import warnings
from typing import Dict, Any, Optional, Union, Tuple


def analyze_amplitude(
    audio_file: str,
    rms_window_ms: float = 50.0,
    consider_dc: bool = True,
) -> Dict[str, Any]:
    """
    分析音频文件的振幅和响度信息

    参数:
        audio_file (str): 音频文件路径
        rms_window_ms (float): RMS窗口大小(毫秒)
        consider_dc (bool): 是否考虑DC偏移

    返回:
        Dict[str, Any]: 包含各种振幅和响度测量值的字典
    """
    # 读取音频文件
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        y, sr = librosa.load(audio_file, sr=None, mono=False)
    
    # 确保y是二维数组 [channels, samples]
    if y.ndim == 1:
        y = y.reshape(1, -1)
    
    # 获取bit depth
    sf_info = sf.info(audio_file)
    bit_depth = sf_info.subtype.split('_')[-1]
    if bit_depth.isdigit():
        bit_depth = int(bit_depth)
    else:
        # 估计bit depth
        bit_depth = int(np.ceil(np.log2(np.max(np.abs(y)) * 2)))
    
    # 计算窗口大小(样本数)
    window_size = int(rms_window_ms * sr / 1000)
    
    # 计算最大和最小样本值
    max_sample = np.max(y) * (2**(bit_depth-1) - 1)
    min_sample = np.min(y) * (2**(bit_depth-1) - 1)
    
    # 计算峰值幅度(dB)
    peak_amplitude = 20 * np.log10(np.max(np.abs(y)))
    
    # 计算TP(True Peak) - 过采样检测真实峰值
    y_over = librosa.resample(y, orig_sr=sr, target_sr=sr*4)
    true_peak_amplitude = 20 * np.log10(np.max(np.abs(y_over)))
    
    # 检查可能的剪切样本数量
    clipped_samples = np.sum(np.abs(y) >= 0.99)
    
    # 计算RMS值
    def calculate_rms(signal):
        if consider_dc:
            return np.sqrt(np.mean(signal**2))
        else:
            # 移除DC偏移
            return np.sqrt(np.mean((signal - np.mean(signal))**2))
    
    # 总体RMS
    total_rms = calculate_rms(y)
    total_rms_db = 20 * np.log10(total_rms) if total_rms > 0 else -np.inf
    
    # 分段RMS
    num_windows = y.shape[1] // window_size
    rms_segments = []
    
    for i in range(num_windows):
        segment = y[:, i*window_size:(i+1)*window_size]
        rms = calculate_rms(segment)
        rms_segments.append(rms)
    
    rms_segments = np.array(rms_segments)
    max_rms = np.max(rms_segments)
    min_rms = np.min(rms_segments)
    avg_rms = np.mean(rms_segments)
    
    max_rms_db = 20 * np.log10(max_rms) if max_rms > 0 else -np.inf
    min_rms_db = 20 * np.log10(min_rms) if min_rms > 0 else -np.inf
    avg_rms_db = 20 * np.log10(avg_rms) if avg_rms > 0 else -np.inf
    
    # 计算DC偏移
    dc_offset = np.mean(y) * 100  # 以百分比表示
    
    # 计算动态范围
    dynamic_range = max_rms_db - min_rms_db
    used_dynamic_range = np.ptp(20 * np.log10(np.clip(rms_segments, 1e-10, None)))
    
    # 计算响度 (旧版)
    loudness_old = total_rms_db
    
    # 感知响度 (简化版)
    # 应用A加权过滤器来模拟人耳感知
    y_weighted = librosa.A_weighting(librosa.db_to_power(librosa.amplitude_to_db(y)))
    perceived_loudness_old = 20 * np.log10(np.sqrt(np.mean(y_weighted**2)))
    
    # ITU-R BS.1770-3 响度
    # 简化版实现
    try:
        import pyloudnorm as pyln
        
        # 创建BS.1770-4测量器
        meter = pyln.Meter(sr)
        loudness_lufs = meter.integrated_loudness(y.T)
    except ImportError:
        # 如果没有pyloudnorm，使用简化实现
        # K加权滤波
        y_filtered = librosa.effects.preemphasis(y, coef=0.98)
        loudness_lufs = -0.691 + 10 * np.log10(np.mean(y_filtered**2)) - 10
    
    # 整合所有测量结果
    results = {
        "peak_amplitude": round(peak_amplitude, 2),
        "true_peak_amplitude": round(true_peak_amplitude, 2),
        "max_sample_value": int(max_sample),
        "min_sample_value": int(min_sample),
        "clipped_samples": int(clipped_samples),
        "total_rms_amplitude": round(total_rms_db, 2),
        "max_rms_amplitude": round(max_rms_db, 2),
        "min_rms_amplitude": round(min_rms_db, 2),
        "avg_rms_amplitude": round(avg_rms_db, 2),
        "dc_offset": round(dc_offset, 2),
        "bit_depth": bit_depth,
        "dynamic_range": round(dynamic_range, 2),
        "used_dynamic_range": round(used_dynamic_range, 2),
        "loudness_old": round(loudness_old, 2),
        "perceived_loudness_old": round(perceived_loudness_old, 2),
        "loudness_lufs": round(loudness_lufs, 2),
        "sample_rate": sr,
        "channels": y.shape[0],
        "duration": y.shape[1] / sr,
        "rms_window_ms": rms_window_ms,
        "consider_dc": consider_dc,
    }
    
    return results


def print_amplitude_info(info: Dict[str, Any]) -> None:
    """
    打印音频振幅分析信息

    参数:
        info (Dict[str, Any]): 由analyze_amplitude返回的振幅信息字典
    """
    print("===== 音频振幅分析 =====")
    print(f"峰值幅度:\t{info['peak_amplitude']} dB")
    print(f"实际峰值幅度:\t{info['true_peak_amplitude']}dBTP")
    print(f"最大采样值:\t{info['max_sample_value']}")
    print(f"最小采样值:\t{info['min_sample_value']}")
    print(f"可能的剪断样本:\t{info['clipped_samples']}")
    print(f"总计 RMS 振幅：\t{info['total_rms_amplitude']} dB")
    print(f"最大 RMS 振幅:\t{info['max_rms_amplitude']} dB")
    print(f"最小 RMS 振幅:\t{info['min_rms_amplitude']} dB")
    print(f"平均 RMS 振幅:\t{info['avg_rms_amplitude']} dB")
    print(f"DC 偏移:\t{info['dc_offset']}%")
    print(f"测量位深度:\t{info['bit_depth']}")
    print(f"动态范围:\t{info['dynamic_range']} dB")
    print(f"使用的动态范围:\t{info['used_dynamic_range']} dB")
    print(f"响度（旧版）:\t{info['loudness_old']} dB")
    print(f"感知响度（旧版）:\t{info['perceived_loudness_old']} dB")
    print(f"ITU-R BS.1770-3 响度: {info['loudness_lufs']} LUFS")
    print("\n附加信息:")
    print(f"采样率: {info['sample_rate']} Hz")
    print(f"声道数: {info['channels']}")
    print(f"持续时间: {info['duration']:.2f} 秒")
    print(f"使用 RMS 窗口 {info['rms_window_ms']} 毫秒")
    print(f"考虑 DC = {info['consider_dc']}")
    print("========================") 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
振幅分析模块的测试
"""

import unittest
import os
import numpy as np
import soundfile as sf
import tempfile

from wavx.analysis import amplitude


class TestAmplitudeAnalysis(unittest.TestCase):
    """测试振幅分析功能"""
    
    def setUp(self):
        """创建测试用的音频文件"""
        # 创建一个正弦波测试音频
        self.sr = 44100  # 采样率
        self.duration = 1  # 秒
        self.freq = 440  # Hz
        
        # 生成音频数据
        t = np.linspace(0, self.duration, int(self.sr * self.duration), endpoint=False)
        self.y = 0.5 * np.sin(2 * np.pi * self.freq * t)
        
        # 创建临时文件
        fd, self.temp_file = tempfile.mkstemp(suffix='.wav')
        os.close(fd)
        
        # 保存到.wav文件
        sf.write(self.temp_file, self.y, self.sr, subtype='PCM_16')
    
    def tearDown(self):
        """删除测试用的临时文件"""
        if os.path.exists(self.temp_file):
            os.unlink(self.temp_file)
    
    def test_analyze_amplitude(self):
        """测试振幅分析函数"""
        # 分析音频文件
        results = amplitude.analyze_amplitude(self.temp_file)
        
        # 检查关键测量值是否合理
        self.assertIsInstance(results, dict)
        self.assertIn('peak_amplitude', results)
        self.assertIn('true_peak_amplitude', results)
        self.assertIn('total_rms_amplitude', results)
        
        # 测试峰值幅度是否接近预期值 (0.5 振幅的正弦波 ≈ -6 dB)
        expected_peak_db = 20 * np.log10(0.5)
        self.assertAlmostEqual(results['peak_amplitude'], expected_peak_db, delta=1)
        
        # 测试RMS是否接近预期值 (正弦波RMS = 振幅/√2)
        expected_rms_db = 20 * np.log10(0.5 / np.sqrt(2))
        self.assertAlmostEqual(results['total_rms_amplitude'], expected_rms_db, delta=1)
        
        # 测试采样率是否正确
        self.assertEqual(results['sample_rate'], self.sr)
    
    def test_print_amplitude_info(self):
        """测试打印函数不抛出异常"""
        results = amplitude.analyze_amplitude(self.temp_file)
        try:
            amplitude.print_amplitude_info(results)
        except Exception as e:
            self.fail(f"print_amplitude_info抛出了异常: {e}")


if __name__ == '__main__':
    unittest.main() 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
标准化模块的测试
"""

import unittest
import os
import numpy as np
import soundfile as sf
import tempfile

from wavx.processing import normalization
from wavx.analysis import amplitude


class TestNormalization(unittest.TestCase):
    """测试音频标准化功能"""
    
    def setUp(self):
        """创建测试用的音频文件"""
        # 创建一个正弦波测试音频
        self.sr = 44100  # 采样率
        self.duration = 1  # 秒
        self.freq = 440  # Hz
        
        # 生成音频数据 - 较小的振幅，用于测试放大
        t = np.linspace(0, self.duration, int(self.sr * self.duration), endpoint=False)
        self.y_quiet = 0.1 * np.sin(2 * np.pi * self.freq * t)
        
        # 生成音频数据 - 较大的振幅，用于测试降低
        self.y_loud = 0.8 * np.sin(2 * np.pi * self.freq * t)
        
        # 创建临时文件
        fd, self.quiet_file = tempfile.mkstemp(suffix='.wav')
        os.close(fd)
        fd, self.loud_file = tempfile.mkstemp(suffix='.wav')
        os.close(fd)
        fd, self.output_file = tempfile.mkstemp(suffix='.wav')
        os.close(fd)
        
        # 保存到.wav文件
        sf.write(self.quiet_file, self.y_quiet, self.sr, subtype='PCM_16')
        sf.write(self.loud_file, self.y_loud, self.sr, subtype='PCM_16')
    
    def tearDown(self):
        """删除测试用的临时文件"""
        for file in [self.quiet_file, self.loud_file, self.output_file]:
            if os.path.exists(file):
                os.unlink(file)
    
    def test_normalize_rms_array(self):
        """测试RMS标准化数组函数"""
        # 测试将安静的音频提高到目标电平
        target_rms_db = -20.0
        normalized = normalization.normalize_rms(
            audio_data=self.y_quiet.reshape(1, -1),
            target_rms_db=target_rms_db
        )
        
        # 计算normalized的RMS
        rms = np.sqrt(np.mean(normalized**2))
        normalized_rms_db = 20 * np.log10(rms)
        
        # 检查标准化后的RMS是否接近目标值
        self.assertAlmostEqual(normalized_rms_db, target_rms_db, delta=0.5)
    
    def test_normalize_to_target_quiet(self):
        """测试将安静的音频标准化到指定RMS"""
        target_rms_db = -20.0
        
        # 获取原始RMS
        original_info = amplitude.analyze_amplitude(self.quiet_file)
        original_rms_db = original_info["total_rms_amplitude"]
        
        # 标准化音频
        result = normalization.normalize_to_target(
            input_file=self.quiet_file,
            output_file=self.output_file,
            target_rms_db=target_rms_db
        )
        
        # 检查结果字典
        self.assertEqual(result["input_file"], self.quiet_file)
        self.assertEqual(result["output_file"], self.output_file)
        self.assertEqual(result["target_rms_db"], target_rms_db)
        self.assertAlmostEqual(result["processed_rms_db"], target_rms_db, delta=0.5)
        
        # 确认增益是否正确应用 (安静音频应该被放大)
        self.assertGreater(result["processed_rms_db"], original_rms_db)
    
    def test_normalize_to_target_loud(self):
        """测试将响亮的音频标准化到指定RMS"""
        target_rms_db = -20.0
        
        # 获取原始RMS
        original_info = amplitude.analyze_amplitude(self.loud_file)
        original_rms_db = original_info["total_rms_amplitude"]
        
        # 标准化音频
        result = normalization.normalize_to_target(
            input_file=self.loud_file,
            output_file=self.output_file,
            target_rms_db=target_rms_db
        )
        
        # 检查结果字典
        self.assertEqual(result["input_file"], self.loud_file)
        self.assertEqual(result["output_file"], self.output_file)
        self.assertEqual(result["target_rms_db"], target_rms_db)
        self.assertAlmostEqual(result["processed_rms_db"], target_rms_db, delta=0.5)
        
        # 确认增益是否正确应用 (响亮音频应该被降低)
        self.assertLess(result["processed_rms_db"], original_rms_db)
    
    def test_reference_signals(self):
        """测试参考信号生成"""
        # 测试正弦波
        sine = normalization.reference_sine_wave(
            duration=0.1,
            amplitude=0.5,
            freq=1000,
            sample_rate=44100
        )
        
        # 检查形状和峰值
        self.assertEqual(sine.shape[0], 1)  # 单声道
        self.assertEqual(sine.shape[1], 4410)  # 0.1秒 * 44100 Hz
        self.assertAlmostEqual(np.max(np.abs(sine)), 0.5, delta=0.01)  # 幅度
        
        # 测试方波
        square = normalization.reference_square_wave(
            duration=0.1,
            amplitude=0.5,
            freq=1000,
            sample_rate=44100
        )
        
        # 检查形状和峰值
        self.assertEqual(square.shape[0], 1)  # 单声道
        self.assertEqual(square.shape[1], 4410)  # 0.1秒 * 44100 Hz
        self.assertAlmostEqual(np.max(np.abs(square)), 0.5, delta=0.01)  # 幅度


if __name__ == '__main__':
    unittest.main() 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 音频处理模块

包含各种音频处理和转换功能
"""

# 导入子模块
from . import normalization

# 为方便使用，直接导出一些常用函数
from .normalization import normalize_rms, normalize_to_target 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 音频标准化示例

这个示例展示了如何使用WavX库将音频文件标准化到指定的RMS电平
"""

import os
import sys
import argparse

# 添加父目录到Python路径，以便可以导入wavx
# 注意：安装库后不需要这一步
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import wavx


def main():
    """主函数，解析命令行参数并标准化音频文件"""
    parser = argparse.ArgumentParser(description='将音频文件标准化到指定的RMS电平')
    parser.add_argument('input_file', help='输入音频文件路径')
    parser.add_argument('output_file', help='输出音频文件路径')
    parser.add_argument('--target', '-t', type=float, default=-20.0, help='目标RMS电平(dB FS)，默认为-20dB')
    parser.add_argument('--reference', '-r', choices=['square', 'sine'], default='square', 
                         help='参考信号类型，可选"square"或"sine"，默认为"square"')
    parser.add_argument('--freq', '-f', type=float, default=1000.0, help='参考信号频率(Hz)，默认为1000Hz')
    args = parser.parse_args()
    
    try:
        # 首先分析原始音频文件
        print(f"分析原始音频文件: {args.input_file}")
        original_info = wavx.analysis.amplitude.analyze_amplitude(args.input_file)
        print("原始音频信息:")
        wavx.analysis.amplitude.print_amplitude_info(original_info)
        
        print("\n正在标准化音频...")
        # 标准化音频文件
        normalization_info = wavx.processing.normalization.normalize_to_target(
            input_file=args.input_file,
            output_file=args.output_file,
            target_rms_db=args.target,
            reference_type=args.reference,
            reference_freq=args.freq
        )
        
        # 打印标准化结果
        wavx.processing.normalization.print_normalization_info(normalization_info)
        
        # 分析处理后的音频文件
        print(f"\n分析处理后的音频文件: {args.output_file}")
        processed_info = wavx.analysis.amplitude.analyze_amplitude(args.output_file)
        print("处理后音频信息:")
        wavx.analysis.amplitude.print_amplitude_info(processed_info)
        
    except Exception as e:
        print(f"错误: {e}")
        return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main())


import os
import sys

# 添加当前目录到Python路径，以便可以导入wavx
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import wavx

audio_file = r"C:\Users\admin\Desktop\250228_QUEST\Noise.wav"
amplitude_info = wavx.analysis.amplitude.analyze_amplitude(audio_file)
print(f"总计 RMS 振幅: {amplitude_info['total_rms_amplitude']} dB")
wavx.analysis.amplitude.print_amplitude_info(amplitude_info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os

# 读取 README.md 作为长描述
with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="wavx",
    version="0.1.2",
    description="音频分析和处理工具库",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Chord",
    author_email="chordjiang@gmail.com",
    url="https://github.com/chord-chord/wavx",
    packages=find_packages(),
    install_requires=[
        "numpy>=1.19.0",
        "scipy>=1.5.0",
        "matplotlib>=3.3.0",
        "soundfile>=0.10.3",
        "librosa>=0.8.0",
    ],
    entry_points={
        'console_scripts': [
            'wavx=wavx.cli:main',
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Multimedia :: Sound/Audio :: Analysis",
    ],
    python_requires=">=3.7",
    project_urls={
        "Bug Tracker": "https://github.com/yourusername/wavx/issues",
        "Documentation": "https://github.com/yourusername/wavx/tree/main/docs",
        "Source Code": "https://github.com/yourusername/wavx",
    },
)